%donald
#contents of the main diagram
viewport view
#angle of inclination of the plane and gravitational field strength
real rotangle, gfs
point neutralstart, neutralend, connectionA, connectionB
openshape fixtures
within fixtures{
	openshape plane
	within plane{
		line surface
		point start, end
		real frictioncoefficient
	}
	openshape pulley
	within pulley{
		point centre
		circle wheel
	}
}
openshape restingblock
within restingblock{
	real mass
	point pA, pB, pC, pD, planemid
	line sideAB, sideBC, sideCD, sideDA

}
openshape hangingblock
within hangingblock{
	real mass
	point pA, pB, pC, pD
	line sideAB, sideBC, sideCD, sideDA
}
openshape rope
within rope{
	point pA, pB, pulley
	line lineAP, linePB
	real forceAtoB
	real forceBtoA
}

within fixtures{
	within plane{
		surface = [start, end]
		frictioncoefficient = 0.1
	}
	plane/start = ~/neutralstart
	plane/end = ~/neutralend
	within pulley{
		centre = {530, 530}
		wheel = circle(centre, 10)
	}
}
within restingblock{
	mass = 1.0
	planemid = midpoint([~/neutralstart, ~/neutralend])
	pA = rot((planemid + {-60, 60}), ~/fixtures/pulley/centre, ~/rotangle)
	pB = rot((planemid + {60, 60}), ~/fixtures/pulley/centre, ~/rotangle)
	pC = rot((planemid + {60, 0}), ~/fixtures/pulley/centre, ~/rotangle)
	pD = rot((planemid + {-60, 0}), ~/fixtures/pulley/centre, ~/rotangle)
	sideAB = [pA, pB]
	sideBC = [pB, pC]
	sideCD = [pC, pD]
	sideDA = [pD, pA]
}

within hangingblock{
	pA = (~/fixtures/pulley/centre + {-30, -300})
	pB = (pA + {60,0})
	pC = (pB + {0, -60})
	pD = (pC + {-60, 0})
	sideAB = [pA,pB]
	sideBC = [pB, pC]
	sideCD = [pC, pD]
	sideDA = [pA, pD]
	mass = 1.0
}

within fixtures{
	plane/start = rot(~/neutralstart, pulley/centre, ~/rotangle)
	plane/end = rot(~/neutralend, pulley/centre, ~/rotangle)
}

within rope{
	pulley = ~/fixtures/pulley/centre
	pA = ~/connectionA
	pB = ~/connectionB
	lineAP = [pA, pulley]
	linePB = [pB, pulley]
}

rotangle = pi div 4
neutralstart = {100, 500}
neutralend = {500, 500}
connectionA = midpoint(restingblock/sideBC)
connectionB = midpoint(hangingblock/sideAB)
gfs = 9.80

#The arrow showing the direction in which the block resting on the plane
#moves, if any
viewport accelarrow1

openshape arrow1
within arrow1{
	point middle
	int direction
	openshape body
	within body{
		point default1
		point default2
		point p1
		point p2
		line bodyline
	}
	openshape head
	within head{
		point default3
		point default4
		point p3
		point p4
		point p5
		line lineA
		line lineB
	}
}
within arrow1{
	middle=midpoint([body/default1, body/default2])
	body/p1 = rot(body/default1, middle,~/rotangle)
	body/p2 = rot(body/default2, middle,~/rotangle)
	within body{
		default1 = {0, 500}
		default2 = {1000, 500}
		bodyline = [p1,p2]
	}
	head/p3 = rot(head/default3, middle,~/rotangle)
	head/p4 = rot(head/default4, middle,~/rotangle)
	head/p5 = rot((middle + {(500*direction), 0}),middle,~/rotangle)
	within head{
		default3 = (~/middle + {(250*~/direction), 250})
		default4 = (~/middle + {(250*~/direction), -250})

		lineA = [p3,p5]
		lineB = [p4,p5]
	}
}
#The arrow showing the direction in which the hanging block moves
viewport accelarrow2
openshape arrow2
within arrow2{
	point middle
	int direction
	openshape body
	within body{
		point p1
		point p2
		line bodyline
	}
	openshape head
	within head{
		point p3
		point p4
		point p5
		line lineA
		line lineB
	}
}
within arrow2{
	middle=midpoint([body/p1, body/p2])
	within body{
		p1 = {500, 0}
		p2 = {500, 1000}
		bodyline = [p1,p2]
	}
	head/p5 = (middle + {0, (500*direction)})
	within head{
		p3 = (~/middle + {250, (250*~/direction)})
		p4 = (~/middle + {-250, (250*~/direction)})
		lineA = [p3,p5]
		lineB = [p4,p5]
	}
}

#Physics stuff starts here

%eden
_rope_forceAtoB is (_hangingblock_mass * _gfs);
normalreaction is ((_restingblock_mass * _gfs)*cos(_rotangle));
maxfriction is (_fixtures_plane_frictioncoefficient*normalreaction);
weightalongplane is (-1.0*(_restingblock_mass * _gfs)*sin(_rotangle));
beforefriction is (weightalongplane + _rope_forceAtoB);
frictionsign is (-1.0*sign(beforefriction));
friction is (frictionsign*min(maxfriction,abs(beforefriction)));
rotangledegs is ((acos(cos(_rotangle))/(2*PI))*360);
rotanglestr is "-" // str(rotangledegs);
netforce is beforefriction + friction;
_arrow1_direction is sign(netforce);
_arrow2_direction is -1.0*sign(netforce);
acceleration is (_restingblock_mass != 0)?(abs(netforce)/_restingblock_mass):0.0;


/*scout definitions for the various text boxes, controls, and screen layout*/
%scout
window dia, angleupbutton, angledownbutton, anglereadout, anglelabel, restingmasslabel;
window hangingmasslabel, frictionlabel, restingmassbox, hangingmassbox, frictionbox, arrow1;
window restingaccellabel, restingaccelreadout, timelabel, timebox, restingvelocityreadout;
window restingdistancereadout, hangingaccellabel, hangingaccelreadout, hangingvelocityreadout;
window hangingdistancereadout, arrow2, frictionreadout, normalreactionreadout;
display scr, screq;
string anglestring, restingaccelstring, restingvelocitystring, restingdistancestring;
string hangingaccelstring, hangingvelocitystring, hangingdistancestring;
string frictionstring, normalreactionstring;
integer equilibrium;
dia = {
	box:	[{1,1}, {500,500}],
	pict: "view",
	type: DONALD,
	border:	1
	sensitive: ON
};
arrow1 = {
	box:	[{30, 530},{120,620}],
	pict:	"accelarrow1",
	type:	DONALD,
	border:	0
	sensitive:	OFF
};
arrow2 = {
	box:	[{400, 530},{490,620}],
	pict:	"accelarrow2",
	type:	DONALD,
	border:	0
	sensitive:	OFF
};
restingaccelreadout = {
	frame:	([{150,560},1,strlen(restingaccelstring)]),
	string:	restingaccelstring,
	border:	0
	sensitive:	OFF
};
restingaccellabel = {
	frame:	([{150,530},1,strlen("Resting block:")]),
	string:	"Resting block:",
	border:	0
	sensitive:	OFF
};
restingvelocityreadout = {
	frame:	([{150, 590},1,strlen(restingvelocitystring)]),
	string:	restingvelocitystring,
	border:	0
	sensitive:	OFF
};
restingdistancereadout = {
	frame:	([{150, 620},1,strlen(restingdistancestring)]),
	string:	restingdistancestring,
	border:	0
	sensitive:	OFF
};
hangingaccelreadout = {
	frame:	([{525,560},1,strlen(hangingaccelstring)]),
	string:	hangingaccelstring,
	border:	0
	sensitive:	OFF
};
hangingaccellabel = {
	frame:	([{525,530},1,strlen("Hanging block:")]),
	string:	"Hanging block:",
	border:	0
	sensitive:	OFF
};
hangingvelocityreadout = {
	frame:	([{525, 590},1,strlen(hangingvelocitystring)]),
	string:	hangingvelocitystring,
	border:	0
	sensitive:	OFF
};
hangingdistancereadout = {
	frame:	([{525, 620},1,strlen(hangingdistancestring)]),
	string:	hangingdistancestring,
	border:	0
	sensitive:	OFF
};
angleupbutton = {
	frame:	([{525,50}, 1, 3]),
	string:	" + ",
	border:	1
	sensitive:	ON
};
angledownbutton = {
	frame:	([{525, 110}, 1, 3]),
	string:	" - ",
	border: 	1
	sensitive:	ON
};
restingmasslabel = {
	frame:	([{525, 140}, 1, strlen("Mass of resting block (kg)")]),
	string:	"Mass of resting block (kg)",
	border:	0
	sensitive:	OFF
};
restingmassbox = {
	type: TEXTBOX,
	frame:	([{525, 170}, {10, 1}]),
	string:	"10",
	border:	1
	sensitive:	ON
};
hangingmasslabel = {
	frame:	([{525, 200}, 1, strlen("Mass of hanging block (kg)")]),
	string:	"Mass of hanging block (kg)",
	border:	0
	sensitive:	OFF
};
hangingmassbox = {
	type: TEXTBOX,
	frame:	([{525, 230}, {10, 1}]),
	string:	"10",
	border:	1
	sensitive:	ON
};
frictionlabel = {
	frame:	([{525, 260}, 1, strlen("Coefficient of Friction")]),
	string:	"Coefficient of Friction",
	border:	0
	sensitive:	OFF
};
frictionbox = {
	type:		TEXTBOX,
	frame:	([{525,290},{10,1}]),
	string:	"0.2",
	border:	1
	sensitive:	ON
};
timelabel = {
	frame:	([{525,320},1,strlen("Time (seconds)")]),
	string:	"Time (seconds)",
	border:	0
	sensitive:	OFF
};
timebox = {
	type:		TEXTBOX
	frame:	([{525,350},{10,1}]),
	string:	"0",
	border:	1
	sensitive:	ON
};
frictionreadout = {
	frame:	([{525,380},1,strlen(frictionstring)]),
	string:	frictionstring,
	border:	0
	sensitive:	OFF
};
normalreactionreadout = {
	frame:	([{525,410},1,strlen(normalreactionstring)]),
	string:	normalreactionstring,
	border:	0
	sensitive:	OFF
};
anglereadout = {
	frame:	([{525, 80},1,strlen(anglestring)]),
	string:	anglestring,
	border:	1
	sensitive:	ON
};
anglelabel = {
	frame:	([{525, 25},1,strlen("Angle of incline (degrees)")]),
	string:	"Angle of incline (degrees)",
	border:	0
	sensitive:	ON
};

#Physics calculations for acceleration, velocity and distance
#Assignments to various strings to be displayed by scout
%eden
anglestring is rotanglestr;
restingaccelstring is ("Acceleration: " // str(acceleration) // " m/s/s");
restingmassbox_setText("1");
hangingmassbox_setText("1");
frictionbox_setText("0.2");
timebox_setText("0");
_restingblock_mass is float(restingmassbox_TEXT_1);
_hangingblock_mass is float(hangingmassbox_TEXT_1);
_fixtures_plane_frictioncoefficient is float(frictionbox_TEXT_1);
equilibrium is _arrow1_direction;
timeelapsed is float(timebox_TEXT_1);
restingvelocitystring is ("Velocity: " // str(acceleration*timeelapsed) // " m/s");
restingdistancestring is ("Distance travelled: " // str(acceleration*timeelapsed*timeelapsed/2) // " m");
hangingaccelstring is restingaccelstring;
hangingvelocitystring is restingvelocitystring;
hangingdistancestring is restingdistancestring;
frictionstring is ("Friction: " // str(abs(friction)) // " N");
normalreactionstring is ("Normal Reaction: " // str(abs(normalreaction)) //" N");

%scout
#Construction of the actual screen.

screq=< dia / angleupbutton / angledownbutton / anglereadout / anglelabel / restingmasslabel / restingmassbox / hangingmasslabel 
		/ hangingmassbox / frictionlabel / frictionbox / restingaccellabel / restingaccelreadout / timelabel / timebox 
		/ restingvelocityreadout / restingdistancereadout / hangingaccellabel / hangingaccelreadout / hangingdistancereadout /
		hangingvelocityreadout / frictionreadout / normalreactionreadout>;

#Don't display the arrows if the system is in equilibrium, since they look odd

scr = if (equilibrium==0) then
	screq
else
	append(screq, 1, arrow1)
endif;

screen = if (equilibrium==0) then
	screq
else
	append(scr, 1, arrow2)
endif;


%eden
/*Deals with mouse click events on the angle adjustment buttons
Should increment the angle by 5 if you hold control but this seems to 
not work all the time, not sure why.*/
proc handle_user_input : input {
	auto newrotangle;
	switch(input) {
		case 1:	newrotangle = _rotangle - ((2*PI)/360);
				if (cos(newrotangle)>0) _rotangle = newrotangle;
				break;
		case 2:	newrotangle = _rotangle + ((2*PI)/360);
				if (cos(newrotangle)>0) _rotangle = newrotangle;
				break;
		case 3:	newrotangle = _rotangle - (((2*PI)/360)*5);
				if (cos(newrotangle)>0) _rotangle = newrotangle;
				break;
		case 4:	newrotangle = _rotangle + (((2*PI)/360)*5);
				if (cos(newrotangle)>0) _rotangle = newrotangle;
				break;

	}
}

proc angleupbutton_to_input : angleupbutton_mouse_1{
	if(angleupbutton_mouse_1[2]==4){
		if(angleupbutton_mouse_1[3]==4) {input = 3;} else {input = 1;}
	}
}

proc angledownbutton_to_input : angledownbutton_mouse_1{
	if(angledownbutton_mouse_1[2]==4){
		if(angledownbutton_mouse_1[3]==4) {input = 4;} else {input = 2;}
	}
}
